@@ -25,7 +25,16 @@
 
 import json
 import re
-import requests
+
+try:
+    # For Python 3.0 and later
+    from urllib.error import HTTPError
+    import urllib.request as urllib
+except ImportError:
+    # Fall back to Python 2's urllib2
+    import urllib2 as urllib
+    from urllib2 import HTTPError
+
 
 url = 'https://review.openstack.org/projects/'
 
@@ -48,21 +57,34 @@ def is_in_openstack_namespace(proj):
 
 
 def has_tempest_plugin(proj):
-    r = requests.get(
-        "https://git.openstack.org/cgit/%s/plain/setup.cfg" % proj)
-    p = re.compile('^tempest\.test_plugins', re.M)
-    if p.findall(r.text):
+    try:
+        r = urllib.urlopen(
+            "https://git.openstack.org/cgit/%s/plain/setup.cfg" % proj)
+    except HTTPError as err:
+        if err.code == 404:
+            return False
+    p = re.compile(r'^tempest\.test_plugins', re.M)
+    if p.findall(r.read().decode('utf-8')):
         return True
     else:
         False
 
-r = requests.get(url)
+
+r = urllib.urlopen(url)
 # Gerrit prepends 4 garbage octets to the JSON, in order to counter
 # cross-site scripting attacks.  Therefore we must discard it so the
 # json library won't choke.
-projects = sorted(filter(is_in_openstack_namespace, json.loads(r.text[4:])))
+projects = sorted(filter(is_in_openstack_namespace, json.loads(r.read()[4:])))
 
-found_plugins = filter(has_tempest_plugin, projects)
+# Retrieve projects having no deb, puppet, ui or spec namespace as those
+# namespaces do not contains tempest plugins.
+projects_list = [i for i in projects if not (
+    i.startswith('openstack/deb-') or
+    i.startswith('openstack/puppet-') or
+    i.endswith('-ui') or
+    i.endswith('-specs'))]
+
+found_plugins = list(filter(has_tempest_plugin, projects_list))
 
 # Every element of the found_plugins list begins with "openstack/".
 # We drop those initial 10 octets when printing the list.
